/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.approvalWorkflow;

import java.io.InputStream;
import java.util.Collection;

import javax.security.auth.login.LoginContext;
import javax.security.auth.login.LoginException;

import org.ow2.bonita.deployment.PackageKey;
import org.ow2.bonita.facade.ManagementAPI;
import org.ow2.bonita.facade.QueryRuntimeAPI;
import org.ow2.bonita.facade.RuntimeAPI;
import org.ow2.bonita.facade.runtime.TaskInst;
import org.ow2.bonita.util.Constants.TaskState;
import org.ow2.novabpm.util.Misc;
import org.ow2.novabpm.util.SimpleCallbackHandler;

/**
 * @author "Pierre Vigneras", "Marc Blachon", "Charles Souillard"
 */
public class ApprovalWorkflow {
  public static final String PROCESS_ID = "ApprovalWorkflow";
  private LoginContext loginContext;

  public void execute(RuntimeAPI runtimeAPI, QueryRuntimeAPI queryRuntimeAPI,
      ManagementAPI managementAPI, String barFile) throws Exception {
    String packageId = deploy(managementAPI, barFile).getPackageId();
    String instanceId = launch(runtimeAPI, queryRuntimeAPI);
    runtimeAPI.deleteInstance(instanceId);
    managementAPI.undeploy(packageId);
  }

  public void login() throws LoginException {
    String loginMode = System.getProperty("login.mode");
    if (loginMode != null && "test".equals(loginMode)) {
      loginContext = new LoginContext("Bonita", new SimpleCallbackHandler("John", "bonita"));
    } else {
      loginContext = new LoginContext("Bonita", new StandardCallbackHandler());
    }
    loginContext.login();
  }
  public void logout() throws LoginException {
    loginContext.logout();
  }

  protected String launch(RuntimeAPI runtimeAPI, QueryRuntimeAPI queryRuntimeAPI) throws Exception {
    final String instanceId = runtimeAPI.instantiateProcess(PROCESS_ID);
    final Collection<TaskInst> tasks = queryRuntimeAPI.getTaskList(instanceId, TaskState.READY);

    if (tasks.isEmpty()) {
      throw new Exception("No task found? Bad User?");
    }
    for (TaskInst task : tasks) {
      runtimeAPI.startTask(task.getId());
      System.out.println("Starting task: " + task.getId());
      runtimeAPI.finishTask(task.getId());
      System.out.println("Task " + task.getId() + " finished.");
    }
    return instanceId;
  }

  protected PackageKey deploy(ManagementAPI managementAPI, String barFile) throws Exception {
    if (barFile == null) {
      System.err.println("Usage: " + ApprovalWorkflow.class + " bpar file");
      System.exit(1);
    }
    final InputStream in = ApprovalWorkflow.class.getResourceAsStream(barFile);
    if (in == null) {
      throw new Exception("Can't find ressource: " + barFile);
    }
    byte[] bar = Misc.getAllContentFrom(in);
    return managementAPI.deployBar(bar);
  }

}
