/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jbpm.pvm.impl;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import org.jbpm.pvm.Execution;
import org.jbpm.pvm.Node;
import org.jbpm.pvm.ProcessDefinition;

/**
 * @author Tom Baeyens
 */
public class ProcessDefinitionImpl extends CompositeElementImpl implements ProcessDefinition {

  private static final long serialVersionUID = 1L;
  private static final Logger log = Logger.getLogger(ProcessDefinitionImpl.class.getName());

  protected String packageName;
  protected int version;
  protected Date deploymentTime;
  protected NodeImpl initial;

  public ProcessDefinitionImpl() {
    this.processDefinition = this;
  }

  // execution factory method /////////////////////////////////////////////////
  
  public Execution startExecution() {
    return startExecution(null, null);
  }
  
  public Execution startExecution(Map<String, Object> variables) {
    return startExecution(null, variables);
  }
  
  public Execution startExecution(String key) {
    return startExecution(key, null);
  }

  public Execution startExecution(String key, Map<String, Object> variables) {
    ExecutionImpl processInstance = newProcessInstance();
    processInstance.setProcess(this);
    processInstance.setProcessInstance(processInstance);
    processInstance.setKey(key);
    processInstance.setNode(initial);
    initializeVariables(processInstance, variables);
    processInstance.start();
    return processInstance;
  }

  protected ExecutionImpl newProcessInstance() {
    log.finest("creating new execution for process "+name);
    return new ExecutionImpl();
  }

  protected void initializeVariables(ExecutionImpl processInstance, Map<String, Object> variables) {
    if (hasVariableDefinitions) {
      processInstance.createVariableScope((List)variableDefinitions);
    } else if (variables!=null) {
      processInstance.createVariableScope();
    }
    processInstance.setVariables(variables);
  }

  // basic methods ////////////////////////////////////////////////////////////

  public String toString() {
    return (name!=null ? "processDefinition("+name+")" : "processDefinition");
  }
  
  // getters and setters //////////////////////////////////////////////////////
  
  public Node getInitial() {
    return initial;
  }
  public void setInitial(NodeImpl initial) {
    this.initial = initial;
  }
  public int getVersion() {
    return version;
  }
  public void setVersion(int version) {
    this.version = version;
  }
  public Date getDeploymentTime() {
    return deploymentTime;
  }
  public void setDeploymentTime(Date deploymentTime) {
    this.deploymentTime = deploymentTime;
  }
  public String getPackageName() {
    return packageName;
  }
  public void setPackageName(String packageName) {
    this.packageName = packageName;
  }
}
