/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.novabpm.identity;

import java.security.Permission;
import java.util.Properties;
import java.util.Set;

/**
 * @author "Pierre Vigneras"
 * @date Nov 23, 2007
 */
public abstract class UserTest extends AbstractTest {

  protected abstract UserOp newUser();

  /**
   * Test method for {@link org.ow2.novabpm.identity.UserOp#getPreferences()}.
   */
  public void testGetId() {
    final UserOp userOp = newUser();
    assertNotNull(userOp.getId());
  }

  //	
  // public void testPreferences() {
  // final UserOp user = newUser();
  // final Preferences preferences = user.getPreferences();
  // assertNotNull(preferences);
  // final String KEY = "preferences";
  // preferences.put(KEY, "NotOk");
  // // Check that the returned preferences is
  // assertEquals(user.getPreferences().get(KEY, "Ok"));
  // }

  /**
   * Test method for {@link org.ow2.novabpm.identity.UserOp#getProperties()}.
   */
  public void testProperties() {
    final UserOp userOp = newUser();
    final Properties properties = userOp.getProperties();
    assertNotNull(properties);
    final String KEY = "properties";
    properties.put(KEY, "A");
    // Check that the returned properties is a copy of the actual one.
    assertEquals("getProperties() should return a copy!", "B", userOp
        .getProperties().getProperty(KEY, "B"));
    userOp.setProperties(properties);
    assertEquals("A", userOp.getProperties().getProperty(KEY, "B"));
  }

  /**
   * Test method for {@link org.ow2.novabpm.identity.UserOp#getPermissions()}.
   */
  public void testPermissions() {
    final UserOp userOp = newUser();
    Set<Permission> permissions = userOp.getPermissions();
    assertNotNull(permissions);
    final Permission permission = new DummyPermission();
    permissions.add(permission);
    assertFalse("Returned permissions should be a copy!", userOp
        .getPermissions().contains(permission));
    userOp.setPermissions(permissions);
    assertTrue("Can't find the added permission!", userOp.getPermissions()
        .contains(permission));
  }

  //
  // /**
  // * Test method for {@link org.ow2.novabpm.identity.UserOp#addPermission()}.
  // */
  // public void testAddPermission() {
  // final Permission permission = new AllPermission();
  // final UserOp user = newUser(Misc.getUniqueId(this.getClass().getName()));
  // final Set<Permission> permissions = user.getPermissions();
  // assertFalse(permissions.contains(permission));
  // user.addPermission(permission);
  // // TODO: Should it be a copy?
  // assertTrue(permissions.contains(permission));
  // }

  /**
   * Test method for {@link org.ow2.novabpm.identity.UserOp#getMemberships()}.
   */
  public void testMemberships() {
    final UserOp userOp = newUser();
    final Set<Membership> memberships = userOp.getMemberships();
    assertNotNull(memberships);
    testUnmodifiableSet(memberships, null);
  }
}
